<?php
//  ------------------------------------------------------------------------ //
//                XOOPS - PHP Content Management System                      //
//                    Copyright (c) 2000 XOOPS.org                           //
//                       <http://www.xoops.org/>                             //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //
// Author: Kazumi Ono (AKA onokazu)                                          //
// URL: http://www.myweb.ne.jp/, http://www.xoops.org/, http://jp.xoops.org/ //
// Project: The XOOPS Project                                                //
// ------------------------------------------------------------------------- //
/**
 * Establishes database class and connection
 *
 * @category	ICMS
 * @package		Database
 *
 * @copyright   The ImpressCMS Project <http://www.impresscms.org>
 * @license		http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU General Public License (GPL)
 */

/**
 *
 * @category	ICMS
 * @package		Database
 *
 * @copyright	copyright (c) 2000-2007 XOOPS.org
 * @copyright   The ImpressCMS Project <http://www.impresscms.org>
 *
 * @abstract
 */
abstract class icms_db_Factory {

	/**
	 * PDO database adapter
	 *
	 * @copyright	The ImpressCMS Project <http://www.impresscms.org>
	 * @var 		icms_db_IConnection
	 */
	static protected $pdoInstance = false;
	
	/**
	 * Legacy database adapter
	 * @var icms_db_legacy_Database
	 */
	static protected $xoopsInstance = false;

	/**
	 * Instanciate the PDO compatible DB adapter (if appropriate).
	 *
	 * @copyright	The ImpressCMS Project <http://www.impresscms.org>
	 *
	 * @throws RuntimeException
	 */
	static public function pdoInstance() {
		if ( self::$pdoInstance !== false ) return self::$pdoInstance;
		if (substr(XOOPS_DB_TYPE, 0, 4) != 'pdo.') return self::$pdoInstance = null;
		if (!class_exists('PDO', false)) {
			throw new RuntimeException("PDO extension not available.");
		}
		$driver = substr(XOOPS_DB_TYPE, 4);
		$dsn = $driver . ':' . XOOPS_DB_DSN;
		$class = "icms_db_{$driver}_Connection";
		if (!class_exists($class)) {
			$class = "icms_db_Connection";
		}
		return self::$pdoInstance = new $class($dsn, XOOPS_DB_USER, XOOPS_DB_PASS);
	}
	/**
	 * Get a reference to the only instance of database class and connects to DB
	 *
	 * if the class has not been instantiated yet, this will also take
	 * care of that
	 *
	 * @copyright	copyright (c) 2000-2007 XOOPS.org
	 * @author		modified by arcandier, The ImpressCMS Project
	 *
	 * @static
	 * @return      object  Reference to the only instance of database class
	 */
	static public function instance() {
		if ( self::$xoopsInstance !== false ) return self::$xoopsInstance;
		$allowWebChanges = defined('XOOPS_DB_PROXY') ? false : true;
		if (substr(XOOPS_DB_TYPE, 0, 4) == 'pdo.') {
			self::$xoopsInstance = new icms_db_legacy_PdoDatabase(self::$pdoInstance, $allowWebChanges);
		} else {
			if (defined('XOOPS_DB_ALTERNATIVE') && class_exists(XOOPS_DB_ALTERNATIVE)) {
				$class = XOOPS_DB_ALTERNATIVE;
			} else {
				$class = 'icms_db_legacy_' . XOOPS_DB_TYPE;
				$class .= $allowWebChanges ? '_Safe' : '_Proxy';
			}
			self::$xoopsInstance = new $class();
			self::$xoopsInstance->setLogger(icms::$logger);
			if (!self::$xoopsInstance->connect()) {
				icms_loadLanguageFile('core', 'core');
				trigger_error(_CORE_DB_NOTRACEDB, E_USER_ERROR);
			}
		}
		self::$xoopsInstance->setPrefix(XOOPS_DB_PREFIX);
		return self::$xoopsInstance;
	}
}
